function [P,xyGrids] = discreteSVT(lambda,rho,sigmaU,sigmaE,Ny,Nx)

%% Compute some uncondtional moments

sigmaX = (sigmaE^2)/(1-rho^2); % unconditional variance of variance process
xBar = 2*log(sigmaU)-sigmaX/2; % unconditional mean of variance process, targeted to match a mean standard deviation of sigmaU
sigmaY = sqrt(exp(xBar+sigmaX/2)/(1-lambda^2)); % uncondtional standard deviation of technology shock

%% Construct technology process approximation

nSigmaX = sqrt(Nx-1);
xGrid = linspace(-nSigmaX*sqrt(sigmaX),nSigmaX*sqrt(sigmaX),Nx) + xBar;
Px = rouwenhorst(Nx,(1+rho)/2,(1+rho)/2);

options = optimoptions(@fmincon,'TolFun',1e-15);
sigmaSpacing = fmincon(@(m) (sqrt(tauchenVarianceSV(m,sigmaY,xBar,lambda,rho,sigmaE,Nx,Ny,Px,xGrid))-sigmaY).^2,2,[],[],[],[],0.1,200,[],options);
yGrid = linspace(-sigmaSpacing*sigmaY,sigmaSpacing*sigmaY,Ny);
yBounds = [-Inf, mean([yGrid(1:end-1); yGrid(2:end)]), Inf];

[~,P,xyGrids] = tauchenVarianceSV(sigmaSpacing,sigmaY,xBar,lambda,rho,sigmaE,Nx,Ny,Px,xGrid);

% Nm = Nx*Ny; % total number of state variable pairs
% xyGrids = flipud(combvec(xGrid,yGrid))';
% P = zeros(Nm);
% for ii = 1:Nm
%     
%     temp = normcdf(yBounds(2:end),lambda*xyGrids(ii,1),sqrt(exp((1-rho)*xBar+rho*xyGrids(ii,2)+(sigmaE^2)/2))) - normcdf(yBounds(1:end-1),lambda*xyGrids(ii,1),sqrt(exp((1-rho)*xBar+rho*xyGrids(ii,2)+(sigmaE^2)/2)));
%     P(ii,:) = kron(temp,ones(1,Nx));
%     P(ii,:) = P(ii,:).*repmat(Px(mod(ii-1,Nx)+1,:),1,Ny);
%     
% end

end